local AddOn = LibStub("AceAddon-3.0"):GetAddon("EquivalencePoints")
local L = LibStub("AceLocale-3.0"):GetLocale("EquivalencePoints")

local rawset, setmetatable = rawset, setmetatable
local remove, insert = table.remove, table.insert

-- Create special table so it looks pretty in Ace3 Config :)
-- http://lua-users.org/wiki/OrderedTableSimple
local function OrderedTableSimple(tbl)
	local mt = {}
   -- set methods
   mt.__index = {
      -- set key order table inside __index for faster lookup
      _korder = {},
      -- traversal of hidden values
      hidden = function() return pairs( mt.__index ) end,
      -- traversal of table ordered: returning index, key
      ipairs = function( self ) return ipairs( self._korder ) end,
      -- traversal of table
      pairs = function( self ) return pairs( self ) end,
      -- traversal of table ordered: returning key,value
      opairs = function( self )
         local i = 0
         local function iter( self )
            i = i + 1
            local k = self._korder[i]
            if k then
               return k,self[k]
            end
         end
         return iter,self
      end,
      -- to be able to delete entries we must write a delete function
      del = function( self,key )
         if self[key] then
            self[key] = nil
            for i,k in ipairs( self._korder ) do
               if k == key then
                  remove( self._korder, i )
                  return
               end
            end
         end
      end,
   }
   -- set new index handling
   mt.__newindex = function( self,k,v )
      if k ~= "del" and v then
         rawset( self,k,v )
         insert( self._korder, k )
      end      
   end
   return setmetatable(tbl or {}, mt)
end

local EPValues = OrderedTableSimple() -- Make this a muppet table with opairs so we can make sure we get values the exact same order as we insert em!
EPValues["EMPTY_SOCKET_META"] = 0 -- Meta gem Bonus

EPValues["STR"] = 0
EPValues["AGI"] = 0
EPValues["STA"] = 0
EPValues["INT"] = 0
EPValues["SPI"] = 0

EPValues["ATTACKPOWER"] = 0
EPValues["CR_HIT"] = 0
EPValues["CR_CRIT"] = 0
EPValues["CR_HASTE"] = 0

EPValues["RANGEDATTACKPOWER"] = 0
--EPValues["CR_RANGEDHIT"] = 0
EPValues["RANGEDCRIT"] = 0
--EPValues["CR_RANGEDHASTE"] = 0
--CR_RANGEDCRIT

EPValues["CR_EXPERTISE"] = 0
EPValues["CR_ARMOR_PENETRATION"] = 0

EPValues["THREATREDUCTION"] = 0

EPValues["SPELLPOWER"] = 0
EPValues["SPELLPEN"] = 0
EPValues["MANAREG"] = 0
EPValues["HOLYCRIT"] = 0
EPValues["CR_RESILIENCE"] = 0

EPValues["ARMOR"] = 0
EPValues["DEFENSE"] = 0
EPValues["CR_DEFENSE"] = 0
EPValues["CR_DODGE"] = 0
EPValues["CR_PARRY"] = 0
EPValues["CR_BLOCK"] = 0
EPValues["BLOCKVALUE"] = 0
EPValues["HEALTHREG"] = 0
--EPValues["MELEEDMG"] = 0
-- MH_DPS = 9.03,
-- OH_DPS = 3.7,

local defaults = {
	profile = {
		Values = EPValues,
		ValueSet = {},
		HiddenSet = {},
		HackExpertise = true,
		ShowZero = true,
		ShowNoBonuses = false,
		MatchRarity = true,
		MaxRarity = 4,
		MaxAvailRarity = 3,
		Consumables = true,
		ProcsAndUse = true,
		ListGems = true,
		ListSetGems = false,
		ClassSpecifics = true,
		ClassSpecificsColor = true,
		RaceSpecifics = true,
		EmptyLineAbove = true,
		EmptyLineBelow = false,
		ReceiveValues = false,
		-- I'm forgetting a setting I've been thinking of countless times so this is a reminder, here they are:
		-- Ignore Hit
		-- Ignore Hit in Sets
		-- Secondary Line with no hit
		Color = {
			r = (127 / 255),
			g = (255 / 255),
			b = (212 / 255),
			a = 1,
		},
		-- All preset values taken from http://www.lootrank.com/wow/templates.asp as of 19/11/2008
		Presets = {
			[L["_preset Death Knight, PvE"]] = { STR = 1.7, STA = 0.3, AGI = 1.0, CR_HASTE = 0.4, CR_CRIT = 1.6, CR_HIT = 1.2, ATTACKPOWER = 0.7, CR_ARMOR_PENETRATION = 1.4, CR_EXPERTISE = 1.2, },
			[L["_preset Death Knight, Tank"]] = { STR = 0.9, STA = 1.06, AGI = 1.02, EMPTY_SOCKET_META = 35, ARMOR = 0.055, CR_DEFENSE = 1.24, CR_DODGE = 1.36, CR_PARRY = 1.09, CR_HASTE = 0.6, CR_CRIT = 0.4, CR_HIT = 0.6, ATTACKPOWER = 0.4, CR_ARMOR_PENETRATION = 0.07, CR_EXPERTISE = 0.9 },
			[L["_preset Druid, Feral Bear"]] = { STA = 4.33, AGI = 6.79, ARMOR = 0.59, CR_DEFENSE = 2.79, CR_DODGE = 4.43, CR_EXPERTISE = 1.69 },
			[L["_preset Druid, Feral Cat"]] = { STR = 2.266, AGI = 2.812, CR_HASTE = 1.443, CR_CRIT = 1.676, CR_HIT = 2.07, ATTACKPOWER = 1, CR_ARMOR_PENETRATION = 2.247, CR_EXPERTISE = 2.096, },
			[L["_preset Druid, Restoration"]] = { MANAREG = 2.4, INT = 0.6, SPELLPOWER = 1.88, STA = 0.4, SPI = 0.8, },
			[L["_preset Hunter, PvE"]] = { AGI = 2.8, INT = 1.1, CR_HASTE = 1.9, RANGEDCRIT = 2.3,  RANGEDATTACKPOWER = 3.0, ATTACKPOWER = 3.0, CR_HIT = 3.1, CR_CRIT = 2.3, CR_ARMOR_PENETRATION = 0.1, MANAREG = 1.0, },
			[L["_preset Mage, Fire, PvE"]] = { STA = 0.4, INT = 1.0, SPI = 0.2, CR_HASTE = 2.4, CR_CRIT = 1.9, CR_HIT = 1.9, MANAREG = 0.5, SPELLPOWER = 2.2, },
			[L["_preset Mage, Frost, PvE 1"]] = { STA = 0.3, INT = 1.0, SPI = 0.2, EMPTY_SOCKET_META = 38, CR_HASTE = 2.4, CR_CRIT = 1.46, CR_HIT = 2.5, SPELLPOWER = 2.2, },
			[L["_preset Mage, Frost, PvE 2"]] = { STA = 0.3, INT = 1.0, SPI = 0.2, EMPTY_SOCKET_META = 38, CR_HASTE = 2.4, CR_CRIT = 1.46, SPELLPOWER = 2.2, },
			[L["_preset Paladin, Holy, PvE"]] = { STA = 0.8, INT = 2.25, ARMOR = 0.1, CR_HASTE = 1, CR_CRIT = 2.25, SPELLPOWER = 2.6, MANAREG = 4.5, },
			[L["_preset Paladin, Protection"]] = { STA = 7.8, AGI = 6.5, INT = 2, EMPTY_SOCKET_META = 18, ARMOR = 0.6, CR_DEFENSE = 8, CR_DODGE = 7.5, BLOCKVALUE = 2, CR_BLOCK = 5, CR_PARRY = 7, CR_HIT = 1.5,  CR_EXPERTISE = 2, SPELLPOWER = 3, },
			[L["_preset Priest, Disc/Holy, PvP"]] = { STA = 1.45, INT = 0.7, SPI = 0.15, EMPTY_SOCKET_META = 20.8, CR_RESILIENCE = 2.32, CR_HASTE = 0.81, CR_CRIT = 0.17, MANAREG = 1, SPELLPOWER = 0.12, },
			[L["_preset Priest, Holy/Disc, PvE 1"]] = { INT = 0.53, SPI = 0.65, EMPTY_SOCKET_META = 17.8, CR_HASTE = 0.1, CR_CRIT = 0.06, MANAREG = 1, SPELLPOWER = 0.51, },
			[L["_preset Priest, Holy/Disc, PvE 2"]] = { INT = 0.49, SPI = 0.62, EMPTY_SOCKET_META = 17.6, CR_HASTE = 0.03, CR_CRIT = 0.06, MANAREG = 1, SPELLPOWER = 0.45, },
			[L["_preset Priest, Shadow, PvE 1"]] = { INT = 0.05, SPI = 0.11, EMPTY_SOCKET_META = 14.7, CR_HASTE = 0.86, CR_CRIT = 0.16, CR_HIT = 1.19, SPELLPOWER = 1, },
			[L["_preset Priest, Shadow, PvE 2"]] = { INT = 0.05, SPI = 0.11, EMPTY_SOCKET_META = 14.75, CR_HASTE = 0.97, CR_CRIT = 0.17, SPELLPOWER = 1, },
			[L["_preset Priest, Shadow, PvP"]] = { STA = 1.5, INT = 0.07, EMPTY_SOCKET_META = 27, CR_HASTE = 0.81, CR_CRIT = 0.17, CR_RESILIENCE = 2, MANAREG = 0.2, SPELLPOWER = 1, },
			[L["_preset Rogue"]] = { STR = 0.55, STA = 0.01, AGI = 1.1, CR_HIT = 1.15, CR_HASTE = 1, CR_EXPERTISE = 1.25, CR_CRIT = 0.9, ATTACKPOWER = 0.5, CR_ARMOR_PENETRATION = 1.05, },
			[L["_preset Shaman, Elemental"]] = { INT = 1.2, CR_HASTE = 3.1, CR_CRIT = 3.9, CR_HIT = 3, MANAREG = 1.5, SPELLPOWER = 5, },
			[L["_preset Shaman, Enhancement"]] = { STR = 1, AGI = 1.85, INT = 1.23, EMPTY_SOCKET_META = 75, CR_CRIT = 2.17, CR_HIT = 3.67, CR_HASTE = 1.71, ATTACKPOWER = 1, CR_ARMOR_PENETRATION = 1.60, CR_EXPERTISE = 3.25, },
			[L["_preset Shaman, Restoration"]] = { STA = 0.2, INT = 1.1, CR_HASTE = 0.5, CR_CRIT = 0.3, MANAREG = 5, SPELLPOWER = 2.068, },
			[L["_preset Warlock, Affliction"]] = { STA = 1, INT = 0.2, CR_HASTE = 0.5, CR_CRIT = 1, CR_HIT = 1.5, SPELLPOWER = 2.5, },
			[L["_preset Warlock, Destruction"]] = { STA = 1, INT = 0.2, CR_HASTE = 2.5, CR_CRIT = 2, CR_HIT = 1.5, SPELLPOWER = 2.5, },
			[L["_preset Warrior, DPS"]] = { STR = 1.6, STA = 0.3, AGI = 1.5, EMPTY_SOCKET_META = 18, CR_HASTE = 2.8, CR_CRIT = 2, CR_HIT = 1.5, CR_EXPERTISE = 1.5, ATTACKPOWER = 0.7, CR_ARMOR_PENETRATION = 1.4, },
			[L["_preset Warrior, Protection"]] = { STR = 2, STA = 7.8, AGI = 6.5, EMPTY_SOCKET_META = 18, ARMOR = 0.6, CR_DEFENSE = 8, CR_DODGE = 7.5, BLOCKVALUE = 4.5, CR_BLOCK = 0.6, CR_PARRY = 7, CR_HIT = 3, CR_EXPERTISE = 6, },
		},
	},
}

function AddOn:DefaultValues()
	-- use 'for key, value in self:DefaultValues() do' to get items in the order they where/are inserted
	return EPValues
end

function AddOn:RegisterDB()
	self.db = LibStub("AceDB-3.0"):New("EquivalencePointsDB", defaults)
end